@@ -27,6 +27,8 @@
 #include <unistd.h>
 #include <errno.h>
 #include <math.h>
+#include <stdlib.h>
+#include <dirent.h>
 #include "lib.h"
 #include "rapl_interface.h"
 
@@ -68,7 +70,11 @@
 #define PP0_DOMAIN_PRESENT	0x04
 #define PP1_DOMAIN_PRESENT	0x08
 
-c_rapl_interface::c_rapl_interface(int cpu) :
+c_rapl_interface::c_rapl_interface(const char *dev_name, int cpu) :
+	powercap_sysfs_present(false),
+	powercap_core_path(),
+	powercap_uncore_path(),
+	powercap_dram_path(),
 	measurment_interval(def_sampling_interval),
 	first_cpu(cpu),
 	last_pkg_energy_status(0.0),
@@ -78,10 +84,63 @@ c_rapl_interface::c_rapl_interface(int cpu) :
 {
 	uint64_t value;
 	int ret;
+	string package_path;
+	DIR *dir;
+	struct dirent *entry;
 
 	RAPL_INFO_PRINT("RAPL device for cpu %d\n", cpu);
 
 	rapl_domains = 0;
+
+	if (dev_name) {
+		string base_path = "/sys/class/powercap/intel-rapl/";
+		if ((dir = opendir(base_path.c_str())) != NULL) {
+			while ((entry = readdir(dir)) != NULL) {
+				string path = base_path + entry->d_name + "/name";
+				string str = read_sysfs_string(path);
+				if (str.length() > 0) {
+					if (str == dev_name) {
+						package_path = base_path + entry->d_name + "/";
+						powercap_sysfs_present = true;
+						rapl_domains |= PKG_DOMAIN_PRESENT;
+						break;
+					}
+				}
+			}
+			closedir(dir);
+		}
+	}
+
+	if (powercap_sysfs_present) {
+		if ((dir = opendir(package_path.c_str())) != NULL) {
+			while ((entry = readdir(dir)) != NULL) {
+				string path = package_path + entry->d_name;
+				string str = read_sysfs_string(path + "/name");
+				if (str.length() > 0) {
+					if (str == "core") {
+						rapl_domains |= PP0_DOMAIN_PRESENT;
+						powercap_core_path = path + "/";
+					}
+					else if (str == "dram") {
+						rapl_domains |= DRAM_DOMAIN_PRESENT;
+						powercap_dram_path = path + "/";
+					}
+					else if (str == "uncore") {
+						rapl_domains |= PP1_DOMAIN_PRESENT;
+						powercap_uncore_path = path + "/";
+					}
+				}
+			}
+			closedir(dir);
+		}
+
+		RAPL_INFO_PRINT("RAPL Using PowerCap Sysfs : Domain Mask %x\n",
+														rapl_domains);
+		return;
+	}
+
+	// Fallback to using MSRs
+
 	// presence of each domain
 	// Check presence of PKG domain
 	ret = read_msr(first_cpu, MSR_PKG_ENERY_STATUS, &value);
@@ -312,6 +371,16 @@ int c_rapl_interface::get_dram_energy_status(double *status)
 		return -1;
 	}
 
+	if (powercap_sysfs_present) {
+		string str = read_sysfs_string(powercap_dram_path + "energy_uj");
+		if (str.length() > 0) {
+			*status =  atof(str.c_str()) / 1000000; // uj to Js
+			return 0;
+		}
+
+		return -EINVAL;
+	}
+
 	ret = read_msr(first_cpu, MSR_DRAM_ENERY_STATUS, &value);
 	if(ret < 0)
 	{
@@ -393,6 +462,16 @@ int c_rapl_interface::get_pp0_energy_status(double *status)
 		return -1;
 	}
 
+	if (powercap_sysfs_present) {
+		string str = read_sysfs_string(powercap_core_path + "energy_uj");
+		if (str.length() > 0) {
+			*status = atof(str.c_str()) / 1000000; // uj to Js
+			return 0;
+		}
+
+		return -EINVAL;
+	}
+
 	ret = read_msr(first_cpu, MSR_PP0_ENERY_STATUS, &value);
 	if(ret < 0)
 	{
@@ -471,6 +550,16 @@ int c_rapl_interface::get_pp1_energy_status(double *status)
 		return -1;
 	}
 
+	if (powercap_sysfs_present) {
+		string str = read_sysfs_string(powercap_uncore_path + "energy_uj");
+		if (str.length() > 0) {
+			*status =  atof(str.c_str()) / 1000000; // uj to Js
+			return 0;
+		}
+
+		return -EINVAL;
+	}
+
 	ret = read_msr(first_cpu, MSR_PP1_ENERY_STATUS, &value);
 	if(ret < 0)
 	{
@@ -558,9 +647,9 @@ void c_rapl_interface::rapl_measure_energy()
 	double pp0_joules = 0;
 	double pp1_joules = 0;
 
-	ret = get_pkg_power_info(&thermal_spec_power, &max_power, &min_power, &max_time_window);
+	get_pkg_power_info(&thermal_spec_power, &max_power, &min_power, &max_time_window);
 	RAPL_DBG_PRINT("Pkg Power Info: Thermal spec %f watts, max %f watts, min %f watts, max time window %f seconds\n", thermal_spec_power, max_power, min_power, max_time_window);
-	ret = get_dram_power_info(&thermal_spec_power, &max_power, &min_power, &max_time_window);
+	get_dram_power_info(&thermal_spec_power, &max_power, &min_power, &max_time_window);
 	RAPL_DBG_PRINT("DRAM Power Info: Thermal spec %f watts, max %f watts, min %f watts, max time window %f seconds\n", thermal_spec_power, max_power, min_power, max_time_window);
 
 	for (;;) {
