@@ -33,7 +33,11 @@
 #include "config.h"
 #endif
 
+#ifdef ENABLE_NLS
 #define _(STRING)    gettext(STRING)
+#else
+#define _(STRING)    (STRING)
+#endif
 
 #define POWERTOP_VERSION "v" PACKAGE_VERSION
 #define POWERTOP_SHORT_VERSION PACKAGE_VERSION
@@ -79,5 +83,7 @@ extern int write_msr(int cpu, uint64_t offset, uint64_t value);
 
 extern void align_string(char *buffer, size_t min_sz, size_t max_sz);
 
-extern void ui_notify_user(const char *frmt, ...);
+extern void ui_notify_user_ncurses(const char *frmt, ...);
+extern void ui_notify_user_console(const char *frmt, ...);
+extern void (*ui_notify_user) (const char *frmt, ...);
 #endif
