@@ -46,6 +46,8 @@
 static void sort_tunables(void);
 static bool should_clear = false;
 
+class tuning_window *tune_window;
+
 class tuning_window: public tab_window {
 public:
 	virtual void repaint(void);
@@ -80,6 +82,11 @@ void initialize_tuning(void)
 	init_tuning();
 
 	w->cursor_max = all_tunables.size() - 1;
+
+	if (tune_window)
+		delete tune_window;
+
+	tune_window = w;
 }
 
 
@@ -198,7 +205,6 @@ void report_show_tunables(void)
 	unsigned int i;
 	/* three tables; bad, unfixable, good */
 	sort_tunables();
-	string srt_tmp;
 	int idx, rows = 0, cols;
 
 	/* First Table */
@@ -230,7 +236,7 @@ void report_show_tunables(void)
 		init_tune_table_attr(&tune_table_css, rows, cols);
 
 		/* Set array of data in row Major order */
-		string tunable_data[cols * rows];
+		string *tunable_data = new string[cols * rows];
 
 		tunable_data[0]=__("Description");
 		tunable_data[1]=__("Script");
@@ -249,6 +255,7 @@ void report_show_tunables(void)
 		/* Report Output */
 		report.add_title(&title_attr,__("Software Settings in Need of Tuning"));
 		report.add_table(tunable_data, &tune_table_css);
+		delete [] tunable_data;
 	}
 
 	/* Second Table */
@@ -258,7 +265,7 @@ void report_show_tunables(void)
 	init_tune_table_attr(&tune_table_css, rows, cols);
 
 	/* Set array of data in row Major order */
-	string untunable_data[rows];
+	string *untunable_data = new string[rows];
 	untunable_data[0]=__("Description");
 
 	for (i = 0; i < all_untunables.size(); i++)
@@ -267,6 +274,7 @@ void report_show_tunables(void)
 	/* Report Output */
 	report.add_title(&title_attr,__("Untunable Software Issues"));
 	report.add_table(untunable_data, &tune_table_css);
+	delete [] untunable_data;
 
 	/* Third Table */
 	/* Set Table attributes, rows, and cols */
@@ -275,7 +283,7 @@ void report_show_tunables(void)
 	init_std_table_attr(&tune_table_css, rows, cols);
 
 	/* Set array of data in row Major order */
-	string tuned_data[rows];
+	string *tuned_data = new string[rows];
 	tuned_data[0]=__("Description");
 	idx=cols;
 	for (i = 0; i < all_tunables.size(); i++) {
@@ -291,7 +299,7 @@ void report_show_tunables(void)
 	report.add_title(&title_attr,__("Optimal Tuned Software Settings"));
         report.add_table(tuned_data, &tune_table_css);
         report.end_div();
-
+	delete [] tuned_data;
 }
 
 void clear_tuning()
